#pragma rtGlobals=1		// Use modern global access method.

// *************************************************
//
//	2008ConvertDaqfact.ipf
//	Author:Donna Sueper
//	Modified by Ken Aikin
//   Revision Number: 1.08
//	Revision Date: Oct 28, 2008

//	Revision Notes: 1.08
//	In function LoadDaqFact(), put function KillAbsolutelyEverything() back to top of function since otherwise it kills the path that was just created.

//	Revision Notes: 1.07
//	In function LoadDaqFact(), put function KillAbsolutelyEverything() after the prompt for path in case user cancels and doesn't want to wipe out everything.

//	Revision Notes: 1.06
//	Fixed bug in name of Timewave created by CreateXHzSpecial if res isn't an integer. (Substitute "_" for ".")

//	Revision Notes: 1.05
//	Put today's date as default into the date/time prompt for the time shift.

//	Revision Notes: 1.04
//	In function CreateXhzSpecial(), added ability to use liberal folder name for the Xhz data.

//	Revision Notes: 1.03
//	In function GetFilesForXHz() the list of selected waves is now printed one item per line instead of 1 long string that goes past the window width.
//	Added more detailed message in prompt for time correction in LoadDaqFact()
//	Commented out line of diagnostic data in Create1Hz and CreateXHz

//	Revision Notes: 1.02
//	Fixed bug in Make wave command in high-res data folder in function CreateXhzSpecial. (Behaved differently under new Igor version?)
//	Fixed bug in CreateXHzSpecial(). Now calls function Expandit with 1/res since CreateXHzSpecial() defines res in Hz.
//	Same fix in function CreateXhz().

//	Revision Notes: 1.01 
//	Changed maximum length of allowed data in routine CreateOverallTimewaveXHz to 5 days of x hz data + 3600 (5*res*86400 + 3600).

//	Revision Notes: 1.0
//	Code is the same as the 2006ConvertDaqfactVx.ipf
//		except that CreateOverallTimeWave now makes integer time wave. Used floor function of first time. (2006 code actually didn't round this off).
//		Also added option to make data sets at X Hz. (LoadDaqFactCreateXHz, which also required CreateOverallTimeWaveXHz)
//		This will process all waves at the desired resolution.
//		Also, added option to make a sub-set of the data at X Hz. Choose LoadDaqFactParts->CreateXHz. User selects waves to operate on using the data browser.
//		Also, added option to correct for time drift. Prompt for correction in seconds, then as each dataset is loaded, correct Timewave, and leave original time in TimewaveOrig.
//		Put the amount of time shift into the wave note for Timewave for each logging set (DataFolder) as well as root level.
//		Uses new function DoTimeShift()
//
// *************************************************
// Where things can go wrong....
//
// If creating 1 hz data, the max time span for is set for 2 day + 1 hour for 1 second data.
//  This is an arbitrary value set to ~2 days to prevent igor crashing if raw time values get messed up
// and it tries to create 1Hz data for long time periods with lots of channels. For X hz data, this is increased
// to 5 days of x Hz data in routine CreateOverallTimewaveXHz().
//
// Because there are no distinctive file name extensions (.dat or .daqfactory or whatever) the program
// will assume that ALL the files in the raw data folder are daqfactory files.  If you put other files in there,
// there may be trouble. However,  it will automatically remove from consideration 
// finder.dat, icon, ds.store and ws_ftp files
//
// The header file for each data file is helpful.  However, to speed data conversion and concatenation, and to simplify things
// Only the first header file of a data logging set is read in.  Parameters such as the number of columns and column names
// are set from this first header file.  The only exception is when data is logged as ascii - then a prompt will appear
// requesting user verification of this scenario.
//
// Things will not work if you change parameters for data logging sets mid-experiment. 
//
// Do not set up your logging set to record time in "excel time format".
// *************************************************

// *************************************************

Menu "Macros"
End


// *************************************************

Menu "DaqFactConvert"
	"LoadDaqFact" 
	help = {"This will translate DaqFactory Files into separate data folders"}

	"LoadDaqFactCreate1Hz"
	help = {"This will do LoadDaqFact AND create 1Hz data in the root folder"}

	"LoadDaqFactCreateXHz"

	"LoadDaqFactCreate1HzKillOrigs"
	help = {"This will do LoadDaqFact AND create 1Hz data in the root folder AND delete the data set folders"}

	Submenu "LoadDaqFactParts"
		"LoadDaqFact"
		"SetVerboseOn"
		"Create1hz"
		"CreateXHz", GetFilesForXHz()	//first calls the function to get input from DataBrowser, then calls CreateXHzSpecial
		"KillDataSetFolders"
	end
End


// *************************************************

Macro LoadDaqFactMacro()
	LoadDaqFact()
End


// *************************************************

Macro LoadDaqFactCreate1Hz()
	LoadDaqFact()
	 Create1hz()
End


// *************************************************

Macro LoadDaqFactCreateXHz()
	LoadDaqFact()
	 CreateXhz()
End

// *************************************************

Macro LoadDaqFactCreate1HzKillOrigs()
	LoadDaqFact()
	Create1hz()
	KillDataSetFolders()	 
End

// *************************************************

Function GetFilesForXHz()	//Operates on a subset of data to produce x Hz data in another data folder that the user names. Uses the data browser for input.
	variable ind, num
	string thiswave
	
	Print "\r****** Using the DataBrowser, select waves (in logging set) to process at X Hz ******\r"
	String cdfBefore = GetDataFolder(1)	// Save current data folder before.
	Execute "CreateBrowser prompt=\"Choose some non-1Hz waves\", showWaves=1, showVars=0, showStrs=0"
	String cdfAfter = GetDataFolder(1)	// Save current data folder after.
	SetDataFolder cdfBefore					// Restore current data folder.
	SVAR S_BrowserList=S_BrowserList
	NVAR V_Flag=V_Flag
	if(V_Flag==0)
		Print "You cancelled."
	else
//		Printf "You selected: %s\r", S_BrowserList	//Printed list all on one line which was too long to read if many waves selected
		num = ItemsInList(S_BrowserList, ";")
		if (num>0)
			Print "You selected the following "+num2str(num)+" waves:"
			for (ind=0; ind<num; ind+=1)
				thiswave = StringFromList(ind, S_BrowserList, ";")
				print thiswave
			endfor
			CreateXhzSpecial(S_BrowserList)
		else
			Abort "No waves selected. Quiting."
		endif
	endif
End

// *************************************************
// Creates an igor data folder DaqFactConversion; loads and concatenates all files in a folder selected by user.
//  It does no 1 Hz time -aligning or any data manipulation.

Function LoadDaqFact()

	KillAbsolutelyEverything()

	NewPath/q/o /m="Where are the Daqfactory files?"  RawDaqFactPath

	if (V_flag!=0)
		Abort "Loading cancelled.  -  Aborting from LoadDaqFact "		// user selected cancel in the previous prompt.
	endif
	
//	original code for time correction (just enter diff at end)
//	variable/G gTimeDiff	//store value of timeshift in global for later access.
//	variable timeDiff
//	prompt timeDiff, "Enter Actual elapsed time - DF elapsed time: "
//	DoPrompt "Enter time drift", timeDiff

	string DFdatetime1, Actualdatetime1, DFdatetime2, Actualdatetime2,yyyystr,mmstr,ddstr
	variable yyyy,mm,dd
	sscanf secs2date(datetime, -1), "%d/%d/%d", dd, mm, yyyy
	sprintf yyyystr, "%4.4d", yyyy
	sprintf mmstr, "%2.2d", mm
	sprintf ddstr, "%2.2d", dd
	DFdatetime1 = yyyystr+"/"+mmstr+"/"+ddstr+" hh:mm:ss"
	Actualdatetime1 = yyyystr+"/"+mmstr+"/"+ddstr+" hh:mm:ss"
	DFdatetime2 = yyyystr+"/"+mmstr+"/"+ddstr+" hh:mm:ss"
	Actualdatetime2 = yyyystr+"/"+mmstr+"/"+ddstr+" hh:mm:ss"
	prompt DFdatetime1, "enter first DaqFact date/time: " 
	prompt Actualdatetime1, "enter first Actual date/time: " 
	prompt DFdatetime2, "enter second DaqFact date/time: " 
	prompt Actualdatetime2, "enter second Actual date/time: " 
	DoPrompt "Enter 2 pairs of date/times or <Continue> to skip time correction", DFdatetime1, Actualdatetime1, DFdatetime2, Actualdatetime2

	if (V_Flag)
		return -1	// User cancelled
	endif

	PRINT "You entered: ", DFdatetime1, Actualdatetime1, DFdatetime2, Actualdatetime2

	NewDataFolder/O/S root:DaqFactConversion
	
	Make/o/t/n=0 DaqFactFileNames, DaqFactFileNamesData, DaqFactFileNamesHeader
	
	PopulateFileNameTextWave(DaqFactFileNames, "RawDaqFactPath")		// This function in 2006GeneralMacros

	Sort DaqFactFileNames DaqFactFileNames		// unnecessary but nice
 	
 	RemoveMiscBadFileNames(DaqFactFileNames)  // gets rid of system-generated or other files -  it eliminates Icon, DS.Store, and Apple Finder 'dot' .xxx files
 	
 	SeparateHeaderDataFiles(DaqFactFileNames, DaqFactFileNamesData, DaqFactFileNamesHeader)
 	
	string/g DataSetsList = ""
 	
 	DataSetsList = GetDataGroupNames(DaqFactFileNamesData)

 	string DataSetName
 	variable DataSetVar, DataSetNum =ItemsInList( DataSetsList)
  	 
	string DaqFactFileGroupStr
	
	Print "// Starting to load the data sets at time "+time()
	
	for (DataSetVar = 0; DataSetVar < DataSetNum; DataSetVar+=1)
		DataSetName = StringFromList(DataSetVar, DataSetsList )		
		NewDataFolder/S $"root:"+DataSetName
		KillWaves/A/Z			// only kills waves in current data folder
		
		LoadOneDacFactDataSet(DataSetName, DaqFactFileNamesData, DaqFactFileNamesHeader)		
//		if (timeDiff!=0)
		if ((strsearch(DFdatetime1, "hh:mm:ss", 0)==-1)&& (strsearch(DFdatetime2, "hh:mm:ss", 0)==-1))	//user entered times, didn't just click OK with default values
//			DoTimeShift(timeDiff)	//creates TimewaveOrig, and corrects Timewave according to timeDiff
			DoTimeShift(DFdatetime1, Actualdatetime1, DFdatetime2, Actualdatetime2)
		endif
		
	endfor
	
	SetDataFolder root:
	
	Print "// Finished loading the data sets at time "+time()
		
	Execute "CreateBrowser" 
	
End


// *************************************************
// Creates 1 hz files for all data sets.

Function Create1hz()

	NewDataFolder/O/S root:DaqFactConversion
	Make/d/o/n=1 ShrinkToFitTimeWave, ShrinkToFitDataWave

	SetDataFolder root:

	SVAR DataSetsList = $"root:DaqFactConversion:DataSetsList"
	variable verbose = NumVarOrDefault("root:DaqFactConversion:VerboseVar",0)
 	
  	string DataSetName
 	variable DataSetVar, DataSetNum =ItemsInList( DataSetsList)
  	 
	variable i, n, numTime
	string str
	
	CreateOverallTimewave()
	
	WAVE BigTimewave = $"root:timeWave"
	WAVE ShrinkedDataWave = $"root:DaqFactConversion:ShrinkToFitDataWave"
	WAVE ShrinkedTimeWave = $"root:DaqFactConversion:ShrinkToFitTimeWave"
	
	numTime = numpnts(BigTimewave)
	
	for (DataSetVar = 0; DataSetVar < DataSetNum; DataSetVar+=1)
	
		DataSetName = StringFromList(DataSetVar, DataSetsList )		
		print "//********* Expanding to 1 Hz the data in the set named "+DataSetName
		
		SVAR liststr = $"root:"+DataSetName+":DaqFactHeaderColNamesStr"
		n = itemsInList(listStr)
		for(i = 0; i<n; i+=1)
			str = StringFromList(i, listStr)
			if (strsearch(lowerstr(str), "time", 0)< 0)		// don't create 1 Hz  files for TimeHi, TimeLo, etc
				Make/o/n=(numTime) $str
				WAVE littletime = $"root:"+DataSetName+":timeWave"
				WAVE littlewave =  $"root:"+DataSetName+":"+str
				WAVE BigWave =$"root:"+str
				if (   (WaveExists(littletime) ==0) || (WaveExists(littlewave) ==0) || (WaveExists(BigTimewave) ==0)|| (WaveExists(BigWave) ==0) || (WaveExists(ShrinkToFitDataWave) ==0)|| (WaveExists(ShrinkToFitTimeWave) ==0) )
//					Print "// ", WaveExists(littletime), WaveExists(littlewave) , WaveExists(BigTimewave) , WaveExists(BigWave) , WaveExists(ShrinkToFitDataWave) , WaveExists(ShrinkToFitTimeWave)
				else
					ShrinkToFit(littlewave, littletime, ShrinkToFitDataWave, ShrinkToFitTimeWave)
					if (verbose)
						ExpandItPrintMissing(ShrinkToFitTimeWave,ShrinkToFitDataWave,BigTimewave,BigWave, 1)
					else
						ExpandIt(ShrinkToFitTimeWave,ShrinkToFitDataWave,BigTimewave,BigWave, 1)
					endif
					
				endif // expected waves exist in the experiment
			endif  // channel is a time related
			
		endfor // all the  channels in the dataSet
		if (strsearch(lowerStr(DataSetName), "aspec",0)>=0) // It is an aerosol - spectra type of data set
			Make1HzSpectraChannelsOneMatrix(DataSetName)
		endif
	endfor //  all the dataSets
	
	SetDataFolder root:
	
	Execute "CreateBrowser" 
	
End

// *************************************************
// Creates X hz files for all data sets.

Function CreateXhz()
	variable res=1
	prompt res, "Enter desired data resolution (Hz):"
	DoPrompt "Resolution", res

	if (V_Flag)
		return -1	// User canceled
	endif

	NewDataFolder/O/S root:DaqFactConversion
	Make/d/o/n=1 ShrinkToFitTimeWave, ShrinkToFitDataWave

	SetDataFolder root:

	SVAR DataSetsList = $"root:DaqFactConversion:DataSetsList"
	variable verbose = NumVarOrDefault("root:DaqFactConversion:VerboseVar",0)
 	
  	string DataSetName
 	variable DataSetVar, DataSetNum =ItemsInList( DataSetsList)
  	 
	variable i, n, numTime
	string str
	
	CreateOverallTimewaveXHz(res, "root")
	
	WAVE BigTimewave = $"root:timeWave"
	WAVE ShrinkedDataWave = $"root:DaqFactConversion:ShrinkToFitDataWave"
	WAVE ShrinkedTimeWave = $"root:DaqFactConversion:ShrinkToFitTimeWave"
	
	numTime = numpnts(BigTimewave)
	
	for (DataSetVar = 0; DataSetVar < DataSetNum; DataSetVar+=1)
	
		DataSetName = StringFromList(DataSetVar, DataSetsList )		
		print "//********* Expanding to "+num2str(res)+" Hz the data in the set named "+DataSetName
		
		SVAR liststr = $"root:"+DataSetName+":DaqFactHeaderColNamesStr"
		n = itemsInList(listStr)
		for(i = 0; i<n; i+=1)
			str = StringFromList(i, listStr)
			if (strsearch(lowerstr(str), "time", 0)< 0)		// don't create 1 Hz  files for TimeHi, TimeLo, etc
				Make/o/n=(numTime) $str
				WAVE littletime = $"root:"+DataSetName+":timeWave"
				WAVE littlewave =  $"root:"+DataSetName+":"+str
				WAVE BigWave =$"root:"+str
				if (   (WaveExists(littletime) ==0) || (WaveExists(littlewave) ==0) || (WaveExists(BigTimewave) ==0)|| (WaveExists(BigWave) ==0) || (WaveExists(ShrinkToFitDataWave) ==0)|| (WaveExists(ShrinkToFitTimeWave) ==0) )
					Print "Some waves were not found."
					Print "// ", WaveExists(littletime), WaveExists(littlewave) , WaveExists(BigTimewave) , WaveExists(BigWave) , WaveExists(ShrinkToFitDataWave) , WaveExists(ShrinkToFitTimeWave)
				else
					ShrinkToFit(littlewave, littletime, ShrinkToFitDataWave, ShrinkToFitTimeWave)
					if (verbose)
						ExpandItPrintMissing(ShrinkToFitTimeWave,ShrinkToFitDataWave,BigTimewave,BigWave, 1/res)
					else
						ExpandIt(ShrinkToFitTimeWave,ShrinkToFitDataWave,BigTimewave,BigWave, 1/res)
					endif
					
				endif // expected waves exist in the experiment
			endif  // channel is a time related
			
		endfor // all the  channels in the dataSet
		if (strsearch(lowerStr(DataSetName), "aspec",0)>=0) // It is an aerosol - spectra type of data set
			Make1HzSpectraChannelsOneMatrix(DataSetName)
		endif
	endfor //  all the dataSets
	
	SetDataFolder root:
	
	Execute "CreateBrowser" 
	
End

Function CreateXhzSpecial(wlist)	//pass in list of waves to operate on for aligning subset of data on X Hz
	string wlist
	
	variable res=1
	string DFname
	prompt res, "Enter desired data resolution (Hz):"
	prompt DFname, "Name for XHz DataFolder:"
	DoPrompt "Resolution and DataFolder name", res, DFname

	if (V_Flag)
		return -1	// User canceled
	endif

	DoAlert 0, "Results will be stored in Data Folder "+DFname
	NewDataFolder/O root:$(DFname)
	NewDataFolder/O/S root:DaqFactConversion
	Make/d/o/n=1 ShrinkToFitTimeWave, ShrinkToFitDataWave

	SetDataFolder root:

	SVAR DataSetsList = $"root:DaqFactConversion:DataSetsList"
	variable verbose = NumVarOrDefault("root:DaqFactConversion:VerboseVar",0)
 	
  	string DataSetName
 	variable DataSetVar, DataSetNum =ItemsInList( DataSetsList)
  	 
	variable i, n, numTime, pos
	string str, fullwname, wname, ext
	
	CreateOverallTimewaveXHz(res, DFname)
	
	string timeWaveStr
	ext = replacestring(".", num2str(res), "_")+"Hz"	//if needed, replace . with _ to have a well behaved wave name
	timeWaveStr = "root:'"+DFname+"':timeWave_"+ext	//add _xHz to all new waves to distinguish them. Add ' around DFname in case it is a "liberal" name.

	WAVE BigTimewave = $(timeWaveStr)
	WAVE ShrinkedDataWave = $"root:DaqFactConversion:ShrinkToFitDataWave"
	WAVE ShrinkedTimeWave = $"root:DaqFactConversion:ShrinkToFitTimeWave"
	
	numTime = numpnts(BigTimewave)
	
	n = itemsInList(wlist)
	for(i = 0; i<n; i+=1)
		fullwname = StringFromList(i, wlist)
		pos = strsearch(fullwname, ":", strlen(fullwname)-1, 1)
		if (pos!=-1)
			wname = fullwname[pos+1, strlen(fullwname)-1]
		else
			abort "Can't extract wave name from fullwave name"
		endif
		dataSetName =  GetWavesDataFolder($fullwname, 0)
		
		if (strsearch(lowerstr(wname), "time", 0)< 0)		// don't create 1 Hz  files for TimeHi, TimeLo, etc
			Make/o/n=(numTime) root:$(DFname):$(wname+"_"+ext)	//add _xHz to new waves created in new datafolder
			WAVE littletime = $"root:"+DataSetName+":timeWave"
			WAVE littlewave =  $"root:"+DataSetName+":"+wname
			WAVE BigWave =$"root:'"+DFname+"':"+wname+"_"+ext		//add _xHz to refer to new waves created in new datafolder
			if (   (WaveExists(littletime) ==0) || (WaveExists(littlewave) ==0) || (WaveExists(BigTimewave) ==0)|| (WaveExists(BigWave) ==0) || (WaveExists(ShrinkToFitDataWave) ==0)|| (WaveExists(ShrinkToFitTimeWave) ==0) )
				Print "Some waves were not found."
				Print "// ", WaveExists(littletime), WaveExists(littlewave) , WaveExists(BigTimewave) , WaveExists(BigWave) , WaveExists(ShrinkToFitDataWave) , WaveExists(ShrinkToFitTimeWave)
			else
				ShrinkToFit(littlewave, littletime, ShrinkToFitDataWave, ShrinkToFitTimeWave)
				if (verbose)
					ExpandItPrintMissing(ShrinkToFitTimeWave,ShrinkToFitDataWave,BigTimewave,BigWave, 1/res)
				else
					ExpandIt(ShrinkToFitTimeWave,ShrinkToFitDataWave,BigTimewave,BigWave, 1/res)	//1/res because in function CreateXhzSpecial, res is in Hz.
				endif
				
			endif // expected waves exist in the experiment
		endif  // channel is a time related
		
	endfor // all the  channels in the dataSet
	if (strsearch(lowerStr(DataSetName), "aspec",0)>=0) // It is an aerosol - spectra type of data set
		Make1HzSpectraChannelsOneMatrix(DataSetName)
	endif
	
	SetDataFolder root:
	
	Execute "CreateBrowser" 
End

// *************************************************
// Removes x y pairs of data where there are nans in original.

Function ShrinkToFit(dw, tw, shrinkedDataWave, shrinkedTimewave)
	Wave dw, tw, shrinkedDataWave, shrinkedTimewave

	variable n = numpnts(tw)
	
	Redimension/n=(n) shrinkedTimewave, shrinkedDataWave
	
	shrinkedTimewave = tw	
	shrinkedDataWave = dw
	
	RemoveNaNsXY(shrinkedTimewave, shrinkedDataWave)

	if(numpnts(shrinkedTimewave) == 0)
		Redimension/n=(1) shrinkedTimewave, shrinkedDataWave
		shrinkedTimewave = tw[0]
		shrinkedDataWave = nan
	endif
	
End


// *************************************************
//  Kills data folders where the non-1Hz data, the original data resides.

Function KillDataSetFolders()

	SVAR DataSetsList = $"root:DaqFactConversion:DataSetsList"

 	string dataSetName
 	variable dataSetVar, dataSetNum =ItemsInList( dataSetsList)

	for (dataSetVar = 0; dataSetVar < dataSetNum; dataSetVar+=1)
		dataSetName = StringFromList(dataSetVar, DataSetsList)		
		KilldataFolder $"root:"+dataSetName
	endfor
	
End


// *************************************************
//  Simply sets a flag that other functions use to print out diagnostics.

Function SetVerboseOn()

	NewDataFolder/o/s root:DaqFactConversion

	variable/g verboseVar = 1
	
	SetDataFolder root: 

End


// *************************************************
//  Creates a comprehensive timewave from all the data sets.

Function CreateOverallTimewave()

	SetDataFolder root:

	WAVE/T DaqFactFileNamesData = $"root:DaqFactConversion:DaqFactFileNamesData"
	SVAR DataSetsList = $"root:DaqFactConversion:DataSetsList"
 	string dataSetName
 	variable dataSetVar, dataSetNum =ItemsInList( dataSetsList)
	if ((exists("root:gOffset")==2) && (exists("root:gDrift")==2))	//these don't exist if user didn't enter anything for time correction
		NVAR lOffset = root:gOffset, lDrift = root:gDrift
	endif
  	 
	variable i, n = numpnts(DaqFactFileNames)
	variable/d/g startTimeVar = datetime, stopTimeVar = 0
	
	for (dataSetVar = 0; dataSetVar < dataSetNum; dataSetVar+=1)
	
		dataSetName = StringFromList(dataSetVar, DataSetsList)		
		
		WAVE/D Twave =$"root:"+dataSetName+":Timewave"
		
		startTimeVar = min(Twave[0], starttimeVar)
		stopTimeVar = max(Twave[numpnts(Twave)-1], stoptimeVar)
	
	endfor
	
	variable numTimePts = ceil(stopTimeVar) - floor(startTimeVar) +1
	
	if (numTimePts > 2* 86400 + 3600)		// arbitrary
		Print (ceil(stopTimeVar) - floor(startTimeVar) +1)
		Abort "The number of points for the timeWave seems too big "+num2str(numTimePts) +" - Aborting from CreateOverallTimewave"
	else
		Make/o/d/n=(numTimePts) Timewave
		Timewave = floor(startTimeVar) +p
		SetScale d 0,0,"dat", Timewave
		Note/K Timewave	//kill any existing wave note
		if ((exists("root:gOffset")==2) && (exists("root:gDrift")==2))
			Note Timewave, "Applied time correction: offset = "+num2str(lOffset)+" seconds and drift = "+num2str(lDrift)+" (Actual time - DF time)."
		endif
	endif
	
	SetDataFolder root:
	
	Execute "CreateBrowser" 
	
End

Function CreateOverallTimewaveXHz(res, DFstr)	//pass in desired data resolution in (Hz) and the DataFolder string to create waves.
	variable res
	string DFstr
	
	SetDataFolder root:

	string ext
	ext = replacestring(".", num2str(res), "_")+"Hz"	//if needed, replace . with _ to have a well behaved wave name

//	WAVE/T DaqFactFileNamesData = $"root:DaqFactConversion:DaqFactFileNamesData"
	SVAR DataSetsList = $"root:DaqFactConversion:DataSetsList"
	if ((exists("root:gOffset")==2) && (exists("root:gDrift")==2))	//these don't exist if user didn't enter anything for time correction
		NVAR lOffset = root:gOffset, lDrift = root:gDrift
	endif
 	
 	string dataSetName
 	variable dataSetVar, dataSetNum =ItemsInList( dataSetsList)
  	 
	variable i, n = numpnts(DaqFactFileNames)
	variable/d/g startTimeVar = datetime, stopTimeVar = 0
	
	for (dataSetVar = 0; dataSetVar < dataSetNum; dataSetVar+=1)
	
		dataSetName = StringFromList(dataSetVar, DataSetsList)		
		
		WAVE/D Twave =$"root:"+dataSetName+":Timewave"
		
		startTimeVar = min(Twave[0], starttimeVar)
		stopTimeVar = max(Twave[numpnts(Twave)-1], stoptimeVar)
	
	endfor
	
	variable numTimePts = ceil(stopTimeVar) - floor(startTimeVar) +1
	
	string timewaveStr
	if (numTimePts > (5 * res * 86400 + 3600))		// 5 days at x Hz plus 3600.
		Print (ceil(stopTimeVar) - floor(startTimeVar) +1)
		Abort "The number of points for the timeWave seems too big "+num2str(numTimePts) +" - Aborting from CreateOverallTimewave"
	else
		if (cmpstr(DFstr, "root")!=0)
			SetDataFolder root:$(DFstr)	//else, already at root level.
			timewaveStr = "Timewave_"+ext
		else
			timewaveStr = "Timewave"
		endif
		Make/o/d/n= (res * numTimePts) $timewaveStr
		wave tw = $timewaveStr
		tw = floor(startTimeVar) + (p/res)
		SetScale d 0,0,"dat", tw
		Note/K tw	//kill any existing wave note
		if ((exists("root:gOffset")==2) && (exists("root:gDrift")==2))
			Note tw, "Applied time correction: offset = "+num2str(lOffset)+" seconds and drift = "+num2str(lDrift)+" (Actual time - DF time)."
		endif
	endif
	
	SetDataFolder root:
	
	Execute "CreateBrowser" 
	
End


// *************************************************
//  Loads all the data from one data set into an igor data folder.

Function LoadOneDacFactDataSet(DataSetName, DaqFactFileNamesData, DaqFactFileNamesHeader)		
	String DataSetName
	Wave/t DaqFactFileNamesData, DaqFactFileNamesHeader

	variable dataSetStart, dataSetStop
	
	dataSetStart =  FinddataSetStart(DataSetName, DaqFactFileNamesData)
	dataSetStop  =  FinddataSetStop(DataSetName, DaqFactFileNamesData)
	
	variable pos, len
	string tempStr, partialName
	len = strlen(DaqFactFileNamesHeader[dataSetStart])
	pos = strsearch(DaqFactFileNamesHeader[dataSetStart], "_", len ,1)	//search backwards from end to find last _ char
	tempStr = DaqFactFileNamesHeader[dataSetStart]
	partialName = tempStr[0, pos-1]
//	if (strsearch(DaqFactFileNamesHeader[dataSetStart], DataSetName,0)>=0)	// we have header files for this data set	
	if (cmpstr(partialName, DataSetName)==0)	// we have header files for this data set	
		ReadOneDaqFactDataSet(DaqFactFileNamesData[dataSetStart]+".head", dataSetStart, dataSetStop, DaqFactFileNamesData)	
	else
		DoAlert 1, "Perhaps there are no header files for the "+ DataSetName +" logging set - continue trying to load?"
		
		if (V_flag ==1)
			LoadWave/O/Q/A/p=RawDaqFactPath/J/D/W/K=0 DaqFactFileNamesData[DataSetStart]		
			string/g DaqFactHeaderTimeStr = ""
			string/g DaqFactHeaderFileTypeStr = "ASCII"
			string/g DaqFactHeaderColNumsStr =  num2str(ItemsInList(S_waveNames))
			string/g DaqFactHeaderColNamesStr = S_waveNames

			LoadRenameAllASCII("", ItemsInList(S_waveNames), S_waveNames, DataSetStart, DataSetStop, DaqFactFileNamesData)
		endif		
	endif

End


// *************************************************
// Goes through the DaqFactFileNamesData text wave to find the first file of a data set to parse.

Function FindDataSetStart(DataSetName, DaqFactFileNamesData)
	String DataSetName
	Wave/t DaqFactFileNamesData

	variable returnVar, i, n	//, nameSetLen, stringLen
	string fName
	
	variable pos, len	//ka added this 20071017
	string partialName

	returnVar = -1
	n = numpnts(DaqFactFileNamesData)
	
//	nameSetLen = strlen(DataSetName)
	
	for (i = 0; i < n; i +=1)
		fName = DaqFactFileNamesData[i]
		len = strlen(fName)
		pos = strsearch(fName, "_", len, 1)	//search backwards from end to get position of last _ char (do it backwards in case someone uses _ in the dataset name)
		partialName = fName[0, pos-1] //get string up to _ char - new
//		stringLen = strlen(fName)
			
//		if (nameSetLen <= stringLen)	//old method
//			if (cmpstr(DataSetName, fName[0, nameSetLen-1]) == 0)	//old method
		if (cmpstr(DataSetName, partialName)==0)	//new method
			returnVar = i
			break
		endif
//		endif
	endfor	
	
	return returnVar

End


// *************************************************
// Goes through the DaqFactFileNamesData text wave to find the last file of a data set to parse

Function FindDataSetStop(DataSetName, DaqFactFileNamesData)
	String DataSetName
	Wave/t DaqFactFileNamesData

	variable returnVar, i, n	//, nameSetLen, stringLen
	string fName
	
	variable pos, len	//ka added this 20071017
	string partialName

	returnVar = -1
	n = numpnts(DaqFactFileNamesData)
	
//	nameSetLen = strlen(DataSetName)

	for (i = n-1; i >=0 ; i -=1)
		fName = DaqFactFileNamesData[i]
		len = strlen(fName)	//fname is something like "LabJackData_10Hz_00000003"
		pos = strsearch(fName, "_", len, 1)	//search backwards from end to get position of last _ char (do it backwards in case someone uses _ in the dataset name)
		partialName = fName[0, pos-1]	// get the string before the _ char. new
//		stringLen = strlen(fName)
			
//		if (nameSetLen <= stringLen)
//			if (cmpstr(DataSetName, fName[0, nameSetLen-1]) == 0)
		if (cmpstr(DataSetName, partialName)==0)
				returnVar = i
				break
			endif
//		endif
	endfor	
	
	return returnVar

End


// *************************************************
//  Returns a list of all file names without numerical suffixes

Function SeparateHeaderDataFiles(DaqFactFileNames, DaqFactFileNamesData, DaqFactFileNamesHeader)
	Wave/t DaqFactFileNames, DaqFactFileNamesData, DaqFactFileNamesHeader

	variable i=0, n = numpnts(DaqFactFileNames)
	string fName

	if (n == 0)
		Abort "There are no entries in this text wave - " + nameofwave(DawFactFileNames) + "-  Aborting from SeparateHeaderDataFiles"
	endif 

	for (i = 0; i < n; i+=1)
		fName = DaqFactFileNames[i]	
		if (cmpstr(fName, ".DS_Store") == 0)
			DeletePoints x, 1, DaqFactFileNames
		endif
	endfor
	
	n = numpnts(DaqFactFileNames)
	
	Redimension/n=(n) DaqFactFileNamesData, DaqFactFileNamesHeader
	
	variable daqFactFileNamesDataNum=0, daqFactFileNamesHeaderNum=0
	
	for (i = 0; i < n; i+=1)
		fName = DaqFactFileNames[i]	
		if ( strsearch(fName, ".head", 0 ) >= 0)
			DaqFactFileNamesHeader[DaqFactFileNamesHeaderNum]= fName
			daqFactFileNamesHeaderNum+=1
		else
			DaqFactFileNamesData[DaqFactFileNamesDataNum]= fName
			daqFactFileNamesDataNum+=1
		endif
	endfor

	DeletePoints daqFactFileNamesHeaderNum, n, DaqFactFileNamesHeader
	DeletePoints daqFactFileNamesDataNum, n, DaqFactFileNamesData

	Sort DaqFactFileNamesHeader DaqFactFileNamesHeader
	Sort DaqFactFileNamesData DaqFactFileNamesData
	
	if (DaqFactFileNamesHeaderNum != DaqFactFileNamesDataNum)
		Print "//  Warning - the number of header files is not the same as the number of data files."
	endif

End


// *************************************************
//  Returns a list of all file names without numerical suffixes.

Function/s GetDataGroupNames(DaqFactFileNamesData)
	Wave/t DaqFactFileNamesData

	variable i=0, n = numpnts(DaqFactFileNamesData)
	string fName
	string dataSetsList = ""
	
	for (i = 0; i < n; i+=1)
		fName = DaqFactFileNamesData[i]	
		fName = StripNumericalSuffix(fName)
		if (cmpstr(fName[strlen(fName)-1], "_") ==0)		
			fName = fName[0,strlen(fName)-2]		// strip _ suffix
		endif
		if(strsearch(fName ".",0) >0)
			fName = fName[0, strsearch(fName ".",0)-1]
		endif
		if ( WhichListItem(fName,dataSetsList) < 0)
			dataSetsList=fName+";"+dataSetsList
		endif
	endfor
	
	return 	dataSetsList
	
End


// *************************************************
//  Returns a list of all file names without numerical suffixes.

Function RemoveMiscBadFileNames(DaqFactFileNames)
	Wave/t DaqFactFileNames
	
	string badFileNameList = "DS.Store;Finder.dat;Icon;WS_FTP"
	
	string daqFactFileNamesList = TextWave2List(DaqFactFileNames)
		
	String cleanDaqFactFileNamesList
	
	cleanDaqFactFileNamesList =  RemoveFromList(badFileNameList, daqFactFileNamesList)

	List2TextWave(cleanDaqFactFileNamesList, NameOfWave(DaqFactFileNames))
	
End


// *************************************************
// Loads in all raw data files of one data set.

Function ReadOneDaqFactDataSet(fileName, DataSetStart, DataSetStop, DaqFactFileNamesData)
	String fileName
	Variable DataSetStart, DataSetStop
	Wave/t DaqFactFileNamesData

	string/g DaqFactHeaderTimeStr
	string/g DaqFactHeaderFileTypeStr
	string/g DaqFactHeaderColNumsStr
	string/g DaqFactHeaderColNamesStr
	
	string SpecStrPrefix
	
	Print "// Reading header file "+fileName
	
	DaqFactHeaderTimeStr = ReadLineXInTextFile("RawDaqFactpath", fileName, 0)		// something is wrong with the format for the first line - ? for Matt
	DaqFactHeaderFileTypeStr = ReadLineXInTextFile("RawDaqFactpath", fileName, 1)
	DaqFactHeaderColNumsStr = ReadLineXInTextFile("RawDaqFactpath", fileName, 3)
	DaqFactHeaderColNamesStr = ReadLineXInTextFile("RawDaqFactpath", fileName,4)
	
	DaqFactHeaderTimeStr= StringByKey("Time", DaqFactHeaderTimeStr)
	DaqFactHeaderFileTypeStr = StringByKey("Type", DaqFactHeaderFileTypeStr)
	DaqFactHeaderColNumsStr= StringByKey("Number Columns", DaqFactHeaderColNumsStr)
	DaqFactHeaderColNamesStr= StringByKey("Columns", DaqFactHeaderColNamesStr)
	
	DaqFactHeaderColNamesStr = StrSubstitute(" ",DaqFactHeaderColNamesStr,"")
	DaqFactHeaderColNamesStr = StrSubstitute(",",DaqFactHeaderColNamesStr,";")
	DaqFactHeaderColNamesStr = StrSubstitute("\r",DaqFactHeaderColNamesStr,"")
	DaqFactHeaderColNamesStr = StrSubstitute("\n",DaqFactHeaderColNamesStr,"")
	
	variable SpecFlag = strsearch(lowerStr(fileName), "spec", 0)
	if (   (SpecFlag>=0) && (cmpstr(lowerStr(fileName[SpecFlag-1]), "a") !=0)  )
	    	print "Radiometer spectra data exists.  Will commence creating 10 minute igor matricies"
		fileName = fileName[0, strsearch(fileName, ".head", 0 )-1]
		LoadRenameSpecAllULONG(DaqFactHeaderTimeStr, fileName,str2num(DaqFactHeaderColNumsStr), DaqFactHeaderColNamesStr, DataSetStart, DataSetStop, DaqFactFileNamesData )			
					
	elseif (strsearch (DaqFactHeaderFileTypeStr,"32 bit float",0  ) >0)			// single float
		fileName = fileName[0, strsearch(fileName, ".head", 0 )-1]
		LoadRenameAllFLOAT(fileName,str2num(DaqFactHeaderColNumsStr), DaqFactHeaderColNamesStr, DataSetStart, DataSetStop, DaqFactFileNamesData )	
	
	elseif  (strsearch (DaqFactHeaderFileTypeStr,"double",0  ) >0)			// double
		fileName = fileName[0, strsearch(fileName, ".head", 0 )-1]
		LoadRenameAllDOUBLE(fileName,str2num(DaqFactHeaderColNumsStr), DaqFactHeaderColNamesStr, DataSetStart, DataSetStop, DaqFactFileNamesData )		

	elseif  (strsearch (DaqFactHeaderFileTypeStr,"integer",0  ) >0)			// 32 bit unsigned integer
		fileName = fileName[0, strsearch(fileName, ".head", 0 )-1]
		LoadRenameAllULONGS(fileName,str2num(DaqFactHeaderColNumsStr), DaqFactHeaderColNamesStr, DataSetStart, DataSetStop, DaqFactFileNamesData )	

	elseif  (strsearch (DaqFactHeaderFileTypeStr,"ASCII",0  ) >0)			// ascii
		fileName = fileName[0, strsearch(fileName, ".head", 0 )-1]
		LoadRenameAllASCII(fileName,str2num(DaqFactHeaderColNumsStr), DaqFactHeaderColNamesStr, DataSetStart, DataSetStop, DaqFactFileNamesData )	

	elseif (strsearch(fileName, ".head", 0 )>0)
		Print "// The header file below does not seem to give information about a data file.  Continuing with other files."
		Print  " // ------------- "+fileName + " ------------- "

	endif
	
End


// *************************************************
//  Reads a specific line from a file.

Function/s ReadLineXInTextFile(pathName, fileName, lineNum)
	String pathName, fileName
	Variable lineNum

	variable refNum
	
	PathInfo $pathName
	if (V_flag ==0)
		Abort "Aborting from ReadLineXInTextFile - this path doesn't exist " +pathName + " - Aborting from ReadLineXInTextFile"
	endif
	
	Open/r/z=2/p=$pathName refNum as fileName
	
	variable x, err = V_flag
	string LineX, fullPath = S_fileName	
	
	if (err ==-1) 
		Print "// ReadLineXInTextFile cancelled by user."
		return ""
	endif
	
	if (err !=0)
		DoAlert 0, "Error in ReadLineXInTextFile"
		return ""
	endif
		
	FReadLine refNum, LineX
	if (lineNum !=0)
		x=1
		do
			FReadLine refNum, LineX
			x+=1
		while( x< lineNum )		
	endif
	Close refNum	
	
	return LineX
	
End


// *************************************************
//  Loads one raw data file of type ULONG.
// ULONGS have the File type  of "Binary - 32 bit unsigned integers" in the .head files

Function LoadRenameULONGS(fileNameStr, numCols, colNameList)
	String fileNameStr, colNameList
	Variable numCols
	
	if (numCols != ItemsInList(colNameList) )
		abort "the number of columns must be the same as the number of column names in list."
	endif
	
	variable colIndex, n
	string colStr
		
	Print " // **** Loading in "+fileNameStr

	Execute "GBLoadWave/O/Q/P=RawDaqFactPath/B/V/T={96,4}/W=("+num2str(numCols) + ") \""+  fileNameStr + "\""
	
	WAVE w0 = $"wave0"
	WAVE w1 = $"wave1"
	Redimension/d w1
	n = numpnts(w0)
	
	Make/d/o/n=(n) timeWave
	SetScale d 0,0,"dat", timeWave
	w1/=1e9
//	timeWave = (w0 + w1 - 365*4  - 2 )*86400 		Excel time
	timeWave = w0 + w1 + date2secs(1970,1,1) // non excel time

	RenameDaqFactWaves(numCols, colNameList)
	
	ConCatDataSet(numCols, colNameList)
	KillWavesInList(colNameList)
	KillWavesInList("Timewave")

End


// *************************************************
// Loads all raw data files of one data set saved as ULONG.

Function LoadRenameAllULONGS(fileNameStr, numCols, colNameList, DataSetStart, DataSetStop, DaqFactFileNamesData)
	String fileNameStr, colNameList		// fileNameStr no necessessary because name is found by DataSetStart and DaqFactFileNamesData
	Variable numCols,  DataSetStart, DataSetStop
	Wave/t DaqFactFileNamesData
	
	variable i
	string fName
	
	KillWaves/a/z		// assumes we are in the right data folder in igor
	
	for (i = DataSetStart; i <=DataSetStop; i+=1)
		fName = DaqFactFileNamesData[i]
		LoadRenameULONGS(fName, numCols, colNameList)		
	endfor
	
	RenameWavesWithThisSuffix("_ALL", "")
End


// *************************************************
// Loads one raw data files of one data set saved as float.

Function LoadRenameFLOAT(fileNameStr, numCols, colNameList)
	String fileNameStr, colNameList
	Variable numCols

	if (numCols != ItemsInList(colNameList) )
		abort "the number of columns must be the same as the number of column names in list."
	endif
		
	variable colIndex, n
	string colStr

	Print " // **** Loading in "+fileNameStr
	Execute "GBLoadWave/O/Q/P=RawDaqFactPath/B/V/T={2, 4}/W=("+num2str(numCols)+") \""+fileNameStr+ "\""

	WAVE w0 = $"wave0"
	WAVE w1 = $"wave1"
	n = numpnts(w0)
	
	Make/d/o/n=(n) timeWave
	SetScale d 0,0,"dat", timeWave
	
	w0 = floor (w0/1000 + 0.5)*1000
	
//	timeWave = (w0 + w1 - 365*4  - 2 )*86400   //Excel Time = fractional days
	timeWave = w0 + w1 +date2secs(1970,1,1) 		// non-Excel time

	RenameDaqFactWaves(numCols, colNameList)

	ConCatDataSet(numCols, colNameList)
	KillWavesInList(colNameList)
	KillWavesInList("Timewave")

End


// *************************************************
// Loads all raw files of one data set saved as a float.

Function LoadRenameAllFLOAT(fileNameStr, numCols, colNameList, DataSetStart, DataSetStop, DaqFactFileNamesData)
	String fileNameStr, colNameList
	Variable numCols,  DataSetStart, DataSetStop
	Wave/t DaqFactFileNamesData

	variable i
	string fName
	
	KillWaves/a/z		// assumes we are in the right data folder in igor
	
	for (i = DataSetStart; i <=DataSetStop; i+=1)
		fName = DaqFactFileNamesData[i]
		LoadRenameFLOAT(fName, numCols, colNameList)		
	endfor

	RenameWavesWithThisSuffix("_ALL", "")

End


// *************************************************
// Loads one file which was saved as a double.

Function LoadRenameDOUBLE(fileNameStr, numCols, colNameList)
	String fileNameStr, colNameList
	Variable numCols

	if (numCols != ItemsInList(colNameList) )
		Abort "the number of columns must be the same as the number of column names in list. - Aborting from LoadRenameDOUBLE"
	endif
				
	variable colIndex, n
	string colStr

	Print " // **** Loading in "+fileNameStr
	Execute "GBLoadWave/Q/P=RawDaqFactPath /B/V/T={4,4}/W=("+num2str(numCols)+") \""+fileNameStr+ "\""

	WAVE w0 = $"wave0"
	WAVE w1 = $"wave1"
	WAVE w2 = $"wave2"
	n = numpnts(w0)
	
	Make/d/o/n=(n) timeWave 
	SetScale d 0,0,"dat", timeWave
	
	w0 = floor (w0/1e5 + 0.5)*1e5
	w1 = floor (w1 + 0.5)

//	timeWave = (w0 + w1 +w2 - 365*4  - 2 )*86400 		// Excel time = fractional days
	timeWave = w0 + w1 +w2  + date2secs(1970,1,1) 		// non-Excel time
	
	RenameDaqFactWaves(numCols, colNameList)

	ConCatDataSet(numCols, colNameList)
	KillWavesInList(colNameList)
	KillWavesInList("Timewave")

End


// *************************************************
// Loads all files of one data sets saved as doubles.

Function LoadRenameAllDOUBLE(fileNameStr, numCols, colNameList, DataSetStart, DataSetStop, DaqFactFileNamesData)
	String fileNameStr, colNameList
	Variable numCols,  DataSetStart, DataSetStop
	Wave/t DaqFactFileNamesData

	variable i
	string fName
	
	KillWaves/a/z		// assumes we are in the right data folder in igor
	
	for (i = DataSetStart; i <=DataSetStop; i+=1)
		fName = DaqFactFileNamesData[i]
		LoadRenameDOUBLE(fName, numCols, colNameList)		
	endfor

	RenameWavesWithThisSuffix("_ALL", "")

End


// *************************************************
// Loads one file of one data set which was saved as ascii.

Function LoadRenameASCII(fileNameStr, numCols, colNameList)
	String fileNameStr, colNameList
	Variable numCols

	if (numCols != ItemsInList(colNameList) )
		Abort "the number of columns must be the same as the number of column names in list. - Aborting from LoadRenameASCII"
	endif
		
	variable n, colIndex
	string colStr
	
	Print " // **** Loading in "+fileNameStr
	
	LoadWave/O/Q/A/p=RawDaqFactPath/J/D/W/K=0 fileNameStr

	if (ItemsInList(S_waveNames) != numCols)
		Print " // The number of columns in the newly loaded ascii file", V_flag, "  does not equal the number of columns in the data set.", numCols
		Print " // The columns in the newly loaded ascii file are", S_waveNames, "  And the columns expected are ", colNameList
		return 0
	endif
	
	for(colIndex=0;colIndex<numCols; colIndex+=1)
		colStr = StringFromList(colIndex, colNameList, ";")
		if (FindListItem(colStr, S_waveNames) < 0)
			Beep
			Print "// This item isn't found in the newly opened file *"+colStr+ "*"+"should be in *"+S_waveNames+"*"
			return 0
		endif
	endfor
		
	WAVE TheTime= $"TheTime"		// when ascii files are loaded in this way, the waves are already named appropriately
	n = numpnts(TheTime)
	
	Make/d/o/n=(n) timewave
	SetScale d 0,0,"dat", timewave 
	
	timewave = TheTime + date2secs(1970,1,1) 		// non-Excel time
	
	ConCatDataSet(numCols, colNameList)

	KillWavesInList(S_waveNames)
	KillWavesInList("Timewave")

End


// *************************************************
// Loads all files of one data set which was saved as ascii.

Function LoadRenameAllASCII(fileNameStr, numCols, colNameList, DataSetStart, DataSetStop, DaqFactFileNamesData)
	String fileNameStr, colNameList
	Variable numCols,  DataSetStart, DataSetStop
	WAVE/t DaqFactFileNamesData
	
	variable i
	string fName
	
	KillWaves/a/z		// assumes we are in the right data folder in igor
	
	for (i = DataSetStart; i<=DataSetStop; i+=1)
		fName = DaqFactFileNamesData[i]
	
		LoadRenameASCII(fName, numCols, colNameList)		
	endfor

	RenameWavesWithThisSuffix("_ALL", "")

End


// *************************************************
// Renames waves to correspond to the Daqfactory names.

Function RenameDaqFactWaves(numCols, colNameList)
	Variable numCols
	String colNameList

	variable colIndex
	string colStr

	for (colIndex = 0; colIndex < numCols; colIndex+=1)
		colStr = StringFromList(colIndex, colNameList, ";")
		colStr = StrSubstitute("\r", colStr, "") 
		colStr = StrSubstitute("\n", colStr, "") 
		colStr = CleanupName(colStr, 0)
		WAVE wx = $"wave"+num2str(colIndex)
		if (WaveExists(wx) ==1)
			ChangeInfs2Nans(wx)
			Rename wx $colStr
		else
			Print "// Wave"+num2str(colIndex)+ " does not exist "
		endif
	endfor
	
End


// *************************************************
//  Appends one wave of data to the master wave which has the suffix _ALL

Function ConCatDataSet(numCols, colNameList)
	Variable numCols
	String colNameList

	variable colIndex
	string colStr
	
	if (numCols!= ItemsInList(colNameList))
		for (colIndex = 0; colIndex < numCols; colIndex+=1)
			colStr = StringFromList(colIndex, colNameList, ";")
		endfor
		Beep
		Print "// Beware!! The number of columns in the data set ", numCols, " dosen't equal the number of columns that was just loaded ", ItemsInList(colNameList), " ",  colNameList
		return	0	// exit	gracefully
	endif
	
	for (colIndex = 0; colIndex < numCols; colIndex+=1)
		colStr = StringFromList(colIndex, colNameList, ";")
		colStr = StrSubstitute("\r", colStr, "") 
		colStr = StrSubstitute("\n", colStr, "") 
		colStr = CleanupName(colStr, 0)
		if (Exists(colStr) != 1)
			Abort "This wave does not exist - "+colStr + " - Aborting from ConCatDataSet"
		else
			ConcatenateWaves(colStr+"_ALL", colStr)
		endif
	endfor
	
	ConcatenateWaves("Timewave_ALL", "Timewave")

End


// *************************************************
// Loads all files of one data set which was saved as spectra files.

Function LoadRenameSpecAllULONG(HeaderTimeStr, fileNameStr, numCols, colNameList, DataSetStart, DataSetStop, DaqFactFileNamesData)
	String HeaderTimeStr, fileNameStr, colNameList
	Variable numCols,  DataSetStart, DataSetStop
	Wave/t DaqFactFileNamesData

	variable x, i , y, n
	string fName, dummyStr, cmdStr, rootString 
	
	KillWaves/a/z		// assumes we are in the right data folder in igor
	
	PathInfo RawDaqFactPath
	rootString = GetPathRootString(S_Path)
	
	NewPath/c/o RawSpectraPath rootString+"Raw Spectra Minute Matrix:"
	NewPath/c/o globalSpectraPath rootString+"Globals:"

	variable/d/g HeaderStartTime, StartTime, StopTime
	string/g UorDStr
	
	UorDStr = StringFromList(3, colNameList, ";")
	UorDStr = UorDStr[0]
	
	HeaderStartTime = str2num(HeaderTimeStr)  + date2secs(1970,1,1) 
	StartTime  = HeaderStartTime   - mod(HeaderStartTime, 600)
	StopTime = StartTime  + 25*6*(600) 		//  assume that the longest we will go is 25 hours
		
	variable/d/g SpectraSize = numCols - 2 // there are 2 columns for time
	variable/d/g MatrixSpectraMatrixVar, MatrixSpectraTime
	string/g  CurrentOpenMatrixStr

	variable numMinuteMatricies =  ( StopTime - StartTime ) /(60*10)

	Make/n=(numMinuteMatricies)/d/o MatrixSpectraTimeWaves		// holds the time in seconds of the first spectra in a matrix
	Make/n=(numMinuteMatricies)/t/o MatrixSpectraTimeNames

	MatrixSpectraTimeWaves = StartTime + p*60*10

	MatrixSpectraTimeNames = UorDStr+"M"+time2yymmddhhmmss(MatrixSpectraTimeWaves[p])
	
	variable/g/d MatrixIndexVar=0	// holds the position within the waves DMatrixSpectraTimeWaves, etc

	CurrentOpenMatrixStr = UorDStr+"M"+time2yymmddhhmmss(StartTime)
	
	Make/o/n=(SpectraSize,60*10)  $CurrentOpenMatrixStr	
	WAVE CurrentOpenMatrix = $CurrentOpenMatrixStr
	CurrentOpenMatrix = nan
	
	for (i = DataSetStart; i <=DataSetStop; i+=1)

		fileNameStr = DaqFactFileNamesData[i]
		Print " // **** Loading in "+fileNameStr
		Execute "GBLoadWave/O/Q/P=RawDaqFactPath/B/V/T={96,4}/W=("+num2str(numCols)+") \""+fileNameStr + "\""

		WAVE w0 = $"wave0"
		WAVE w1 = $"wave1"
		
		Redimension/d w1
		n = numpnts(w0)
			
		Make/d/o/n=(n) spectraTime 
		Make/o/n=(n, SpectraSize) spectraMatrix

		SetScale d 0,0,"dat", spectraTime
		
		w1/=1e9

		spectraTime = w0 + w1 + date2secs(1970,1,1) // non excel time
		
		for(i = 0; i < SpectraSize; i+=1)
			WAVE wi = $"wave"+num2str(i+2)  // note that the first 2 waves that were loaded in were timeWaves
			if (waveexists(wi) == 0)
				Abort "// The wave doesn't exist  "+"wave"+num2str(i+3) + " - Aborting from LoadRenameSpecAllULONG"
			endif
			spectraMatrix[][i] = wi[p]	
		endfor
		KillWavesInList(WaveList("wave*", ";", "" ) )
		
		MatrixTranspose spectraMatrix
		
		DumpSpectraWaves10MinMatricies(spectraTime, SpectraMatrix, UorDStr)
		DumpSpectraWavesPixel580(SpectraTime, SpectraMatrix, UorDStr)

	endfor

	KillwavesinList("spectraTime;spectraMatrix;temptimeWave")
	
	WAVE CurrentOpenMatrix = $CurrentOpenMatrixStr	
	Save/c/o/p=rawSpectraPath CurrentOpenMatrix as NameofWave(CurrentOpenMatrix)+".ibw"			
	KillWavesInList(CurrentOpenMatrixStr)

	SaveWavesInList("globalSpectraPath", UorDStr+"Pixel580TimeAll;"+UorDStr+"Pixel580WaveAll;")	
			
	WAVE Pixel580TimeAll = $UorDStr+"Pixel580TimeAll"
	
	StopTime = Pixel580TimeAll[numpnts(Pixel580TimeAll)-1]
	StopTime = StopTime - mod(StopTime, 600) + 600

	numMinuteMatricies =  ( StopTime - StartTime ) /(60*10)

	Redimension/N=(numMinuteMatricies) MatrixSpectraTimeNames
	Redimension/N=(numMinuteMatricies) MatrixSpectraTimeWaves

	Rename MatrixSpectraTimeWaves $UorDStr+"MatrixSpectraTimeWaves"
	Rename MatrixSpectraTimeNames $UorDStr+"MatrixSpectraTimeNames"

	SaveWavesInList("globalSpectraPath", UorDStr+"MatrixSpectraTimeNames;"+UorDStr+"MatrixSpectraTimeWaves;")	

End


// *************************************************
// Rearranges raw spectra data into nice 10 minute matrix groups.

Function  DumpSpectraWaves10MinMatricies(SpectraTime, SpectraMatrix, UorDstr)
	WAVE SpectraTime, SpectraMatrix
	String UorDstr

	variable/d ThisInputStart, ThisInputStop, ThisSectionMatrixStart, ThisSectionMatrixStop, CurrentMatrixStart, CurrentMatrixStop
	string pathStr 
		
	CarefullyAdjustTimeNearHalfSec(spectraTime)
			
	ThisInputStart =round(SpectraTime[0])
	ThisInputStop = round(SpectraTime[numpnts(SpectraTime)-1])
	ThisSectionMatrixStart = ThisInputStart - mod(ThisInputStart, 60*10)
	ThisSectionMatrixStop = ThisInputStop - mod(ThisInputStop, 60*10) + 60*10 - 1
	
	Make/o/d/n=(60*10) tempTimewave = ThisSectionMatrixStart+p
		
	NVAR MatrixSpectraMatrixVar = $"MatrixSpectraMatrixVar"
	NVAR MatrixSpectraTime = $"MatrixSpectraTime"
	SVAR CurrentOpenMatrixStr = $"CurrentOpenMatrixStr"
	WAVE CurrentOpenMatrix = $CurrentOpenMatrixStr			
	pathStr = "rawSpectraPath"
	
	if (  (NVAR_Exists(MatrixSpectraMatrixVar)==0)%|(NVAR_Exists(MatrixSpectraTime)==0)%|(SVAR_Exists(CurrentOpenMatrixStr)==0)%|(WaveExists(CurrentOpenMatrix)==0)  )
		Print NVAR_Exists(MatrixSpectraMatrixVar)
		Print NVAR_Exists(MatrixSpectraTime)
		Print SVAR_Exists(CurrentOpenMatrixStr)
		Print CurrentOpenMatrixStr
		Print WaveExists(CurrentOpenMatrix)	
		Abort "Something doesn't exist -  Aborting from DumpSpectraWaves10MinMatricies"
	endif
	
	do
		Print "// ", CurrentOpenMatrixStr,  "start", secs2time(ThisInputStart,3), secs2time(ThisSectionMatrixStart,3), secs2time(ThisInputStop,3), secs2time(ThisSectionMatrixStop,3)

		CurrentMatrixStart = yymmddhhmmss2time(CurrentOpenMatrixStr[2,15])  // DM020711124000 
		CurrentMatrixStop = CurrentMatrixStart + 60*10 - 1

		WAVE CurrentOpenMatrix = $CurrentOpenMatrixStr
		
		if (exists(CurrentOpenMatrixStr)==0)
			Abort "this wave does not exist "+CurrentOpenMatrixStr + " Aborting from DumpSpectraWaves10MinMatricies"
		endif	

		if( (ThisSectionMatrixStart <= CurrentMatrixStart ) )
			DumpInto1HzSpectraMatrix(SpectraTime, SpectraMatrix, tempTimewave, CurrentOpenMatrix)
		endif		
		
		if( CurrentMatrixStop <= ThisInputStop )
			
			Save/o/p=$pathStr CurrentOpenMatrix as NameofWave(CurrentOpenMatrix)+".ibw"			
			
			ThisSectionMatrixStart = yymmddhhmmss2time(CurrentOpenMatrixStr[2,strlen(CurrentOpenMatrixStr)-1])+ 60*10
			ThisSectionMatrixStop = ThisSectionMatrixStart + 60*10 - 1
			
			tempTimewave = ThisSectionMatrixStart+p
			
			CurrentOpenMatrixStr = UorDStr+"M"+time2yymmddhhmmss(ThisSectionMatrixStart )
			if (Exists(CurrentOpenMatrixStr)==1)
				Abort "The following file exists "+CurrentOpenMatrixStr + " - Aborting from DumpSpectraWaves10MinMatricies"
			endif	
			Rename CurrentOpenMatrix $CurrentOpenMatrixStr
			WAVE CurrentOpenMatrix = $CurrentOpenMatrixStr			
			CurrentOpenMatrix = nan
			
		endif
	while ( ThisInputStop > CurrentMatrixStop )

End


// *************************************************
// Adjusts time one half second forward when time nears the half second mark.

Function CarefullyAdjustTimeNearHalfSec(timeWave)
	Wave timeWave

	Duplicate/o timeWave fracSeconds
	fracSeconds = timeWave[p]  - floor(timeWave[p])
	variable midSecondPt = 0

	do
		midSecondPt = findfirstGE(fracSeconds, .5, midSecondPt)
		if (  (fracSeconds[midSecondPt] < .55) %& (midSecondPt < numpnts(fracSeconds)-1 )   ) 		// we are looking for a point that is < .55 - close to the roundoff place 
			Print "We are near the mid-second point.  Added .2 to the time for the next 150 spectra", midSecondPt
			timeWave[midSecondPt,min(midSecondPt+150, numpnts(fracSeconds)-1)]+=.2
			midSecondPt+=150
		endif
		midSecondPt = findfirstLE(fracSeconds, .4, midSecondPt)
	while(  midSecondPt < numpnts(fracSeconds)-1  )
	
	Killwaves fracSeconds
End


// *************************************************
// Dumps one pixel value, 580 into a separate wave for later diagnostic use.

Function  DumpSpectraWavesPixel580(SpectraTime, SpectraMatrix, DownUpstr)
	Wave SpectraTime, SpectraMatrix
	String DownUpstr

	if (  cmpstr(DownUpstr, "D")==0)
		Make/o/n=(numpnts(SpectraTime)) DPixel580wave
		DPixel580wave = SpectraMatrix[580][p]
		ConcatenateWaves("DPixel580TimeAll", NameofWave(SpectraTime))
		ConcatenateWaves("DPixel580waveAll", "DPixel580wave")
	endif
	if (  cmpstr(DownUpstr, "U")==0)
		Make/o/n=(numpnts(SpectraTime)) UPixel580wave
		UPixel580wave = SpectraMatrix[580][p]
		ConcatenateWaves("UPixel580TimeAll", NameofWave(SpectraTime))
		ConcatenateWaves("UPixel580waveAll", "UPixel580wave")
	endif

	KillWavesInList("DPixel580wave;UPixel580wave")
End


// *************************************************
//  Manipulates spectra data to align values with 1 Hz time.

Function DumpInto1HzSpectraMatrix(fileTimeWave, fileSpectraMatrix, timeWave, SpectraMatrix)	// expands to 1 sec
	Wave/d fileTimeWave, fileSpectraMatrix, timeWave, SpectraMatrix

	fileTimeWave = round(fileTimeWave)	
	timeWave = round(timeWave)

	variable x1=0, x2=0
	variable n1=numpnts(fileTimeWave), n2=numpnts(timeWave)
	
	do
		if ( timeWave[x2]==fileTimeWave[x1] )
			SpectraMatrix[][x2] = fileSpectraMatrix[p][x1]
			x1+=1
			x2+=1
	//		print "moved this column "+num2str(x2) 
		else
			if (timeWave[x2] < fileTimeWave[x1] )
				x2+=1
			else
				x1+=1
			endif
		endif
	while ( ( x2 < n2 )%&(x1< n1 ) )

End


// *************************************************
//  Makes a 1 hz matrix out of aerosol spectra channels.

Function Make1HzSpectraChannelsOneMatrix(DataSetName)	
	String DataSetName

	WAVE timeWave = $"root:timeWave"	
	SVAR DaqFactHeaderColNamesStr = $"root:"+DataSetName + ":DaqFactHeaderColNamesStr"

	if ( (WaveExists(timeWave) ==0)||(SVAR_Exists(DaqFactHeaderColNamesStr)==0)  )
		Print "// Error in Make1HzSpectraChannelsOneMatrix - timeWave or DaqFactHeaderColNamesStr doesn't exist."
	else
	
		variable i, n	
		string str, daqFactHeaderColNamesStrNoTime = ""
		
		n = ItemsInList(DaqFactHeaderColNamesStr)
		
		for (i=0; i<n;i+=1)
			str = StringFromList(i, DaqFactHeaderColNamesStr, ";")
			if (strsearch (lowerStr(str), "time", 0)>=0)
			else
				daqFactHeaderColNamesStrNoTime += str+";"
			endif	
		endfor
		
		n = ItemsInList(daqFactHeaderColNamesStrNoTime)
	
		SetDataFolder root:
		
		Make/o/n=(numpnts(timeWave), n) $DataSetName + "Matrix"
		
		WAVE DataSetMatrix =  $DataSetName + "Matrix"
		
		DataSetMatrix = nan
		
		for (i=0; i<n;i+=1)
			str = StringFromList(i, daqFactHeaderColNamesStrNoTime, ";")
			WAVE w = $"root:"+str  // assume that the 1 hz files have been create first
			if (WaveExists(w) ==0)
				Print  "Error in Make1HzSpectraChannelsOneMatrix - wave "+str+" doesn't exist."
				break
			else
				DataSetMatrix[][i] = w[p]
			endif
		endfor
	endif

End

//--------Time correction code -----------------

function oldDoTimeShift(timeDiff)
	variable timeDiff
	
	variable startTime, DFendTime, trueEndTime, len, corr
	wave Timewave
 
 	len = numpnts(Timewave)
	startTime = Timewave[0]		//assume that start times are equal for DF and Actual time
	DFendTime = Timewave[len-1]
	trueEndTime = DFendTime + timeDiff
	corr = (trueEndTime - DFendTime) / (DFendTime - startTime)	//calculate the amount to add to each second. This is the slope.
	
	duplicate/O Timewave, TimewaveOrig
	Timewave += (Timewave-Timewave[0]) * corr
	Note/K Timewave
	Note Timewave, "Applied time shift of "+num2str(timeDiff)+" seconds (Actual time - DF time)."
//	Timewave = Timewave + (p * timeDiff/(len-1))
end

function DoTimeShift(DFdatetime1, Actualdatetime1, DFdatetime2, Actualdatetime2)	//pass in two pairs of DF and true times in format yyyy/mm/dd hh:mm:ss
	string DFdatetime1, Actualdatetime1, DFdatetime2, Actualdatetime2
	
	variable len, offset, drift, DFfirstTime, ActualfirstTimeCalc, DFlastTime, ActualLastTimeCalc
	variable slope, b
	variable DFtime1, ActualTime1, DFtime2, ActualTime2
	variable DFyear1, DFmonth1, DFday1, DFhr1, DFmin1, DFsec1
	variable DFyear2, DFmonth2, DFday2, DFhr2, DFmin2, DFsec2
	variable Actualyear1, Actualmonth1, Actualday1, Actualhr1, Actualmin1, Actualsec1
	variable Actualyear2, Actualmonth2, Actualday2, Actualhr2, Actualmin2, Actualsec2

	wave Timewave
 
	sscanf DFdatetime1, "%d/%d/%d %d:%d:%d", DFyear1, DFmonth1, DFday1, DFhr1, DFmin1, DFsec1
	sscanf ActualDatetime1, "%d/%d/%d %d:%d:%d", Actualyear1, Actualmonth1, Actualday1, Actualhr1, Actualmin1, Actualsec1

	sscanf DFdatetime2, "%d/%d/%d %d:%d:%d", DFyear2, DFmonth2, DFday2, DFhr2, DFmin2, DFsec2
	sscanf ActualDatetime2, "%d/%d/%d %d:%d:%d", Actualyear2, Actualmonth2, Actualday2, Actualhr2, Actualmin2, Actualsec2

//convert these to igor datetime format
	DFtime1 = date2secs(DFyear1, DFmonth1, DFday1) + DFhr1*3600 + DFmin1*60 + DFsec1
	ActualTime1 = date2secs(Actualyear1, Actualmonth1, Actualday1) + Actualhr1*3600 + Actualmin1*60 + Actualsec1
	DFtime2 = date2secs(DFyear2, DFmonth2, DFday2) + DFhr2*3600 + DFmin2*60 + DFsec2
	ActualTime2 = date2secs(Actualyear2, Actualmonth2, Actualday2) + Actualhr2*3600 + Actualmin2*60 + Actualsec2
	
 	len = numpnts(Timewave)
	slope = (DFtime2-DFtime1) / (ActualTime2-ActualTime1)
	print "slope= ", slope

	b = DFtime1 - slope*ActualTime1	//intercept (y=mx+b)

	DFfirstTime = Timewave[0]
// x=(y-b)/m
	ActualfirstTimeCalc = (DFfirstTime - b) / slope
	offset = ActualfirstTimeCalc - DFfirstTime
//	print "offset = ", offset
	
	DFlastTime = Timewave[len-1]
	ActualLastTimeCalc = (DFlastTime - b) / slope

	drift = (ActualLastTimeCalc - DFlastTime) - offset	//total drift in data from start to end
//	print "drift = ", drift

	duplicate/O Timewave, TimewaveORIG
	Timewave += offset	
//	Timewave[1, ] = Timewave + (p*(1- slope))	//no drift on first point [0]		//only works on evenly spaces times
	Timewave[1, ] += (Timewave-Timewave[0]) * (1-slope)	//this accounts for non-evenly spaced times

//	create globals to store the time values just calculated
	variable/G root:gOffset, root:gDrift
	NVAR lOffset = root:gOffset, lDrift = root:gDrift
	lOffset = offset
	lDrift = drift
	
	Note/K Timewave
	Note Timewave, "Applied time correction: offset = "+num2str(offset)+" seconds and drift = "+num2str(drift)+" (Actual time - DF time)."
	Print "// ---- Applied time correction: offset = "+num2str(offset)+" seconds and drift = "+num2str(drift)+" (Actual time - DF time). -----"
end
